from datetime import date, datetime  # noqa: F401
from typing import Dict, List  # noqa: F401

from timestep.api.ap.v1 import util
from timestep.api.ap.v1.models.base_model import Model


class TaskRequestBody(Model):
    """NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).

    Do not edit the class manually.
    """

    def __init__(self, input=None, additional_input=None):  # noqa: E501
        """TaskRequestBody - a model defined in OpenAPI

        :param input: The input of this TaskRequestBody.  # noqa: E501
        :type input: str
        :param additional_input: The additional_input of this TaskRequestBody.  # noqa: E501
        :type additional_input: object
        """
        self.openapi_types = {"input": str, "additional_input": object}

        self.attribute_map = {"input": "input", "additional_input": "additional_input"}

        self._input = input
        self._additional_input = additional_input

    @classmethod
    def from_dict(cls, dikt) -> "TaskRequestBody":
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The TaskRequestBody of this TaskRequestBody.  # noqa: E501
        :rtype: TaskRequestBody
        """
        return util.deserialize_model(dikt, cls)

    @property
    def input(self) -> str:
        """Gets the input of this TaskRequestBody.

        Input prompt for the task.  # noqa: E501

        :return: The input of this TaskRequestBody.
        :rtype: str
        """
        return self._input

    @input.setter
    def input(self, input: str):
        """Sets the input of this TaskRequestBody.

        Input prompt for the task.  # noqa: E501

        :param input: The input of this TaskRequestBody.
        :type input: str
        """

        self._input = input

    @property
    def additional_input(self) -> object:
        """Gets the additional_input of this TaskRequestBody.

        Input parameters for the task. Any value is allowed.  # noqa: E501

        :return: The additional_input of this TaskRequestBody.
        :rtype: object
        """
        return self._additional_input

    @additional_input.setter
    def additional_input(self, additional_input: object):
        """Sets the additional_input of this TaskRequestBody.

        Input parameters for the task. Any value is allowed.  # noqa: E501

        :param additional_input: The additional_input of this TaskRequestBody.
        :type additional_input: object
        """

        self._additional_input = additional_input
